/*
 * SideKickTree.java
 * :tabSize=8:indentSize=8:noTabs=false:
 * :folding=explicit:collapseFolds=1:
 *
 * Copyright (C) 2000, 2003 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/*
Changes:
   JL: 12.4.2004 added a popupmenu to CustomTree and show it upon right button -click
   JL: 13.4.2004 added some functionality to the popupmenu
   JR&RP: 10.5.2004 modified popupmenu, popupmenu is created as defined in property file
   JR: 12.5.2004 popmenus functionality modified, functions are defined in actions.xml
   JR: 14.5.2004 SideKicks action Save as png-image added.
ToDo:
   - dragging and dropping an element (moving elements)

   - upon selection from tree, should we select the area in textarea and move
     the view to the beginning of the selection??

   - Text content edit -window??

 */


package sidekick;

//{{{ Imports
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.Enumeration;
import java.awt.datatransfer.*; // JL: added for clipboard
import java.util.Vector;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.jedit.textarea.*;
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;
import java.util.StringTokenizer;
import javax.xml.parsers.*;
import org.xml.sax.*;
import java.io.StringReader;
import java.io.File; // JR: added for action saving treeview as png
import java.awt.image.BufferedImage; // JR: added for action saving treeview as png
import java.io.FileOutputStream; // JR: added for action saving treeview as png
import java.io.IOException; // JR: added for action saving treeview as png

//}}}

public class SideKickTree extends JPanel implements EBComponent,
DefaultFocusComponent
{

  public final static int BEFORE = 0;
  public final static int INSIDE = 1;
  public final static int AFTER = 2;

	//{{{ SideKickTree constructor
	public SideKickTree(View view, boolean docked)
	{
		super(new BorderLayout());

		this.view = view;

		// create toolbar with parse button
		JToolBar buttonBox = new JToolBar();
		buttonBox.setFloatable(false);

		parseBtn = new RolloverButton(GUIUtilities.loadIcon("Parse.png"));
		parseBtn.setToolTipText(jEdit.getProperty("sidekick-tree.parse"));
		parseBtn.setMargin(new Insets(0,0,0,0));
		parseBtn.setRequestFocusEnabled(false);
		parseBtn.addActionListener(new ActionHandler());
		buttonBox.add(parseBtn);
		buttonBox.add(Box.createGlue());

		add(BorderLayout.NORTH,buttonBox);

		// create a faux model that will do until a real one arrives
		DefaultTreeModel emptyModel = new DefaultTreeModel(
			new DefaultMutableTreeNode(null));
		tree = new CustomTree(emptyModel);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addKeyListener(new KeyHandler());

		if(docked)
			tree.addMouseMotionListener(new MouseHandler());

		// looks bad with the OS X L&F, apparently...
		if(!OperatingSystem.isMacOSLF())
			tree.putClientProperty("JTree.lineStyle", "Angled");

		tree.setVisibleRowCount(30);
		tree.setCellRenderer(new Renderer());


               // Xetor
               contentArea = new JEditorPane();
               contentArea.setMinimumSize(new Dimension(100,100));
               contentArea.setMaximumSize(new Dimension(100,100));
               contentArea.setPreferredSize(new Dimension(100,100));
               contentArea.addFocusListener(new ContentAreaFocusAdapter((CustomTree)tree,contentArea));
               contentArea.addCaretListener(new ContentAreaCaretListener((CustomTree)tree,contentArea));

               treepanel = new JScrollPane(tree);
               tree.addTreeSelectionListener(new MyTreeSelectionListener(tree, contentArea/*, view.getBuffer()*/));

               JSplitPane splitter = new JSplitPane(JSplitPane.VERTICAL_SPLIT, treepanel, new JScrollPane(contentArea));
               splitter.setPreferredSize(new Dimension(100,100));
               splitter.setDividerLocation(0.5);
               splitter.setResizeWeight(0.50);
               splitter.setDividerSize(20);
               add(BorderLayout.CENTER,splitter);

               changeTextButton = new JButton();
               changeTextButton.setPreferredSize(new Dimension(75,25));
               changeTextButton.setText(jEdit.getProperty("sidekick-contentarea.buttonText"));

               changeTextButtonPanel = new JPanel(new BorderLayout());
               changeTextButtonPanel.add(BorderLayout.EAST,changeTextButton);


               add(BorderLayout.SOUTH,changeTextButtonPanel);
               //------


               propertiesChanged();

		CaretHandler caretListener = new CaretHandler();

		EditPane[] editPanes = view.getEditPanes();
		for(int i = 0; i < editPanes.length; i++)
		{
			editPanes[i].getTextArea().addCaretListener(
				caretListener);
		}

		update();
	} //}}}

        public boolean contentAreaHasFocus() {
          return contentArea.isFocusOwner();
        }


        public boolean treeHasFocus() {
          return tree.isFocusOwner();
        }


        public CustomTree getTree() {
          return (CustomTree)tree;
        }

	//{{{ focusOnDefaultComponent() method
	public void focusOnDefaultComponent()
	{
		tree.requestFocus();
	} //}}}

	//{{{ addNotify() method
	public void addNotify()
	{
		super.addNotify();
		EditBus.addToBus(this);
	} //}}}

	//{{{ removeNotify() method
	public void removeNotify()
	{
		super.removeNotify();
		EditBus.removeFromBus(this);
	} //}}}

	//{{{ handleMessage() method
	public void handleMessage(EBMessage msg)
	{
		if(msg instanceof EditPaneUpdate)
		{
			EditPaneUpdate epu = (EditPaneUpdate)msg;
			EditPane editPane = epu.getEditPane();

			if(epu.getWhat() == EditPaneUpdate.CREATED)
				editPane.getTextArea().addCaretListener(new CaretHandler());
		}
		else if(msg instanceof PropertiesChanged)
			propertiesChanged();
		else if(msg instanceof SideKickUpdate)
		{
			if(((SideKickUpdate)msg).getView() == view)
				update();
		}
	} //}}}

        /** Olavi Paananen
          * This method activates tree
          *
        */
        //{{{ activateTree() method
        public void activateTree() {
          tree.setEnabled(true);
          treeFollowsCaret = true;
        }//}}}

        /** Olavi Paananen
         *  This method deActivates the tree
         *  - used when there is errors
         */
        //{{{
        public void deActivateTree() {
          tree.setEnabled(false);
          treeFollowsCaret = false;
        } //}}}


        public void cut() {
          ((CustomTree)tree).copySelectionToClipboard();
          deleteSelection();
        }

        public void copy(){
          ((CustomTree)tree).copySelectionToClipboard();
        }

        public void paste(int where) {
          switch (where) {
            case BEFORE:
              // code to paste clipboard contents before currently selected tag
              ((CustomTree)tree).pasteFromClipboard(true);
              break;

            case INSIDE:
              ((CustomTree)tree).pasteInside();
              break;

            case AFTER:
              ((CustomTree)tree).pasteFromClipboard(false);
              break;
          }
        }

        public void delete() {
          deleteSelection();
        }

        public void sendEditTagMsg() {
          EditBus.send(new ShowEditTagDialog());
        }



        /** OP: This method returns the selected ghost element from the tree
         *      if ghostelement not selected returns null
         */
        //{{{ getSelectedElement() method
        public GhostElement getSelectedElement() {
          GhostElement retval = null;
          TreePath path = tree.getSelectionPath();
          if (path == null) {
            //System.err.print("Tss mennn pihalle koska polku on null.");
            return retval;
          }

          Object obj = ((DefaultMutableTreeNode)path.getLastPathComponent()).getUserObject();
          //System.err.print(obj.toString());
          if ( obj instanceof GhostElement ) retval = (GhostElement)obj;

          return retval;
        }//}}}

        //{{{ Private members

	//{{{ Instance variables
	private RolloverButton parseBtn;
	private JTree tree;

        private JEditorPane contentArea;
        private JPanel changeTextButtonPanel;
        private JButton changeTextButton;

	private boolean treeFollowsCaret;

	private View view;
	private Timer caretTimer;

	private SideKickParsedData data;

	private JFileChooser fc; //14.5.04 JR added for saving treeview as png
	private File file; //14.5.04 JR added for saving treeview as png

	private JScrollPane treepanel;
	//}}}

	//{{{ propertiesChanged() method
	private void propertiesChanged()
	{
		treeFollowsCaret = jEdit.getBooleanProperty("sidekick-tree.follows-caret");
	} //}}}

	//{{{ update() method
	private void update()
	{
		data = SideKickParsedData.getParsedData(view);
		if(SideKickPlugin.getParserForBuffer(view.getBuffer()) == null
			|| data == null)
		{
			DefaultMutableTreeNode root = new DefaultMutableTreeNode(view.getBuffer().getName());
			root.insert(new DefaultMutableTreeNode(
				jEdit.getProperty("sidekick-tree.not-parsed")),0);

			tree.setModel(new DefaultTreeModel(root));
		}
		else
		{
                        // OP: tree should not collapse
                        tree.setModel(data.tree);
                        if(treeFollowsCaret)
				expandTreeAt(view.getTextArea().getCaretPosition());
		}
	} //}}}

	//{{{ expandTreeWithDelay() method
	private void expandTreeWithDelay()
	{
		// if keystroke parse timer is running, do nothing
		// if(keystrokeTimer != null && keystrokeTimer.isRunning())
			// return;

		if(caretTimer != null)
			caretTimer.stop();

		caretTimer = new Timer(0,new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				JEditTextArea textArea = view.getTextArea();
				int caret = textArea.getCaretPosition();
				Selection s = textArea.getSelectionAtOffset(caret);
				expandTreeAt(s == null ? caret : s.getStart());
			}
		});

		caretTimer.setInitialDelay(500);
		caretTimer.setRepeats(false);
		caretTimer.start();
	} //}}}

	//{{{ expandTreeAt() method
	private void expandTreeAt(int dot)
 	{
		if(data == null)
			return;

		TreePath treePath = data.getTreePathForPosition(dot);
		if(treePath != null)
		{
			tree.expandPath(treePath);
			tree.setSelectionPath(treePath);
			tree.scrollPathToVisible(treePath);
		}
	} //}}}


	//{{{ Inner classes

	//{{{ CustomTree class
	public class CustomTree extends JTree
	{
        	private JPopupMenu popup;// JL: added
		private JPopupMenu popupElements; //JR added
		CustomTree(TreeModel model)
		{
			super(model);
			//8.5.04 JR&RP: context menu is created as defined in property file
			popup = GUIUtilities.loadPopupMenu("sidekick-tree.context");
                        //setListenCaretChanges(true);
 		}


                private void pasteFromClipboard(boolean before) {
                  View view = jEdit.getActiveView();
		  TreePath path = getSelectionPath();
                  if(path == null) return;

                  Object value = ((DefaultMutableTreeNode)path
                                  .getLastPathComponent()).getUserObject();
                  if ( !(value instanceof Asset) ) return;

                  Asset asset = (Asset)value;
                  Clipboard clipboard = getToolkit().getSystemClipboard();
                  JEditTextArea textArea = view.getTextArea();

                  Transferable contents = clipboard.getContents(this);

                  // Determine if data is available in string flavor
                  if(contents != null && contents.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                     try {
                       String string;
                       // Have contents cough up string
                       string = (String) contents.getTransferData(
                       DataFlavor.stringFlavor);

                       //--------------JL: Test well-formedness-----------------------------------
                       // idea: try to parse the string to be pasted. if some error occurs during parsing, the string is
                       //       not well-formed xml and some dialog should be shown to the user.

                       // Obtain a SAX parser from a SAX parser factory
                       SAXParserFactory parserFactory = SAXParserFactory.newInstance();
                       parserFactory.setValidating(false);
                       parserFactory.setNamespaceAware(true);
                       SAXParser parser = parserFactory.newSAXParser();

                       org.xml.sax.helpers.DefaultHandler handler = new MyHandler();

                       try {
                         parser.parse(new InputSource(new StringReader(string)), handler);
                       } catch (SAXParseException e) {
                         // show dialog here
                         view.getStatus().setMessage("not well-formed");
                         return;
                       }
                     //----------------------------------------------------------------------------

                       if (before) textArea.getBuffer().insert(asset.start.getOffset(), string);
                       else textArea.getBuffer().insert(asset.end.getOffset(), string);
                     }
                     catch(Exception ex) {
                       ex.printStackTrace();
                     }
                  }
                }

                private void copySelectionToClipboard() {
                     Asset asset = getCurrentAsset();
                     Clipboard clipboard = getToolkit().getSystemClipboard();
                     JEditTextArea textArea = view.getTextArea();

                     // Wrap the data in a transferable object
                     StringSelection contents = new StringSelection(
                       textArea.getSelectedText( new Selection.Range(asset.start.getOffset(), asset.end.getOffset()) )
                     );
                     // Place the transferable onto the clipboard
                     clipboard.setContents(contents,
                       new ClipboardOwner() {
                         public void lostOwnership(Clipboard clip,Transferable transferable) {

                         }
                       }
                     );
                 }


                public boolean hasRoot() {
                  TreeModel model = getModel();
                  return !( model.isLeaf(model.getRoot()) );
                }

                public boolean isRootSelected() {
                  TreePath path = getSelectionPath();
                  if (path==null) view.getStatus().setMessage("isRootSelected() path == null");
                  return (path.getPathCount() == 2); // root selected
                }

                public Asset getCurrentAsset() {
                  TreePath path = getSelectionPath();
                  if(path == null) return null;

                  Object value = ((DefaultMutableTreeNode)path
                                 .getLastPathComponent()).getUserObject();

                  if ( !(value instanceof Asset) ) return null;

                  return (Asset)value;
                }


               public void pasteInside()
               {
                 View view = jEdit.getActiveView();
		 view.getStatus().setMessage("paste inside");

                 TreePath path = getSelectionPath();
                 if(path == null) return;

                 Object value = ((DefaultMutableTreeNode)path
                                 .getLastPathComponent()).getUserObject();
                 if ( !(value instanceof Asset) ) return;

                 Asset asset = (Asset)value;
                 Clipboard clipboard = getToolkit().getSystemClipboard();
                 JEditTextArea textArea = view.getTextArea();

                 Transferable contents = clipboard.getContents(this);

                 // Determine if data is available in string flavor
                 if(contents != null && contents.isDataFlavorSupported(DataFlavor.stringFlavor)) {
                    try {
                      String string;
                      // Have contents cough up string
                      string = (String) contents.getTransferData(
                          DataFlavor.stringFlavor);
                      DefaultMutableTreeNode node = ( (DefaultMutableTreeNode) tree.
                                                     getSelectionPath().
                                                     getLastPathComponent());
                      Object nodeObject = node.getUserObject();
                      if (nodeObject instanceof Asset) {
                        asset = (Asset) nodeObject;
                        if (!jEdit.getActiveView().getBuffer().getText(asset.end.
                            getOffset() - 2, 1).equals("/")) {
                          int offset = 0;
                          if (node.isLeaf()) {
                            offset = asset.end.getOffset() - asset.name.length() -
                                3;
                          }
                          else {
                            Object obj = ( (DefaultMutableTreeNode) node.getFirstChild()).
                                getUserObject();
                            if (obj instanceof Asset) {
                              asset = (Asset) obj;
                              offset = asset.start.getOffset();
                            }
                            if (obj instanceof TextElement) {
                              TextElement textEl = (TextElement) obj;
                              offset = textEl.start;
                            }
                            if (obj instanceof GhostElement) {
                              GhostElement ghost = (GhostElement) obj;
                              offset = ghost.getOffsetStart();
                            }
                          }

                          textArea.getBuffer().insert(offset, string);
                        }
                      }
                    }
                    catch(Exception ex) {
                      ex.printStackTrace();
                    }
                 }
               }


		protected void processMouseEvent(MouseEvent evt)
		{
			// OP: added this so that caret won't follow disenabled tree
                        if ( !isEnabled() ) return;
                        switch(evt.getID())
			{
			//{{{ MOUSE_PRESSED...
			case MouseEvent.MOUSE_PRESSED:
				TreePath path = getPathForLocation(
					evt.getX(),evt.getY());
				if(path != null)
				{
					Object value = ((DefaultMutableTreeNode)path
						.getLastPathComponent()).getUserObject();

					if(value instanceof Asset)
					{
						Asset asset = (Asset)value;
						JEditTextArea textArea = view.getTextArea();

                                                /********************* JL: Added  ******************************/
                                                 if (evt.getButton() == MouseEvent.BUTTON3) {

                                                  // this selects the element clicked and the corresponding area on textarea
                                                   textArea.setCaretPosition(asset.start.getOffset());
                                                   super.processMouseEvent(evt);  //

                                                   // The following is copied from JEditTextArea.java...
                                                   if(GUIUtilities.isPopupTrigger(evt) && popup != null)
                                                   {
                                                           if(popup.isVisible())
                                                                   popup.setVisible(false);
                                                           else
                                                           {
                                                                   GUIUtilities.showPopupMenu(popup,this,
                                                                           evt.getX(),evt.getY());
                                                           }

                                                           return;
                                                   }
                                                 }
                                                 /********************* JL: end of additions      ******************************/


						/* if(evt.getClickCount() == 2)
						{
							textArea.setCaretPosition(tag.start.getOffset());
							expandPath(path);
							XmlActions.showEditTagDialog(view);
							return;
						}
						else */
						if (evt.getButton() == MouseEvent.BUTTON1) {
						  if (evt.isShiftDown()) {
                                                    textArea.setCaretPosition(asset.end.getOffset());
                                                    textArea.addToSelection(
                                                        new Selection.Range(
                                                        asset.start.getOffset(),
                                                        asset.end.getOffset()));
                                                  }
                                                  else if (evt.isControlDown()) {
                                                    textArea.getDisplayManager().narrow(
                                                        textArea.getLineOfOffset(asset.start.getOffset()),
                                                        textArea.getLineOfOffset(asset.end.getOffset()));
                                                  }
                                                  else
                                                    textArea.setCaretPosition(asset.start.getOffset());

                                                  //EditBus.send(new UpdateTagListMsg(view,asset.start.getOffset(),asset.end.getOffset()));
                                                }
					//testailuja
					}
					else if(value instanceof GhostElement){
					  if (evt.getButton() == MouseEvent.BUTTON1) {
					    JEditTextArea textArea = view.getTextArea();
                                            GhostElement ghostelement = (GhostElement)value;
				            view.getStatus().setMessage("t on ghostelement");
					    textArea.setCaretPosition(ghostelement.getOffsetStart());
					    popupElements = GUIUtilities.loadPopupMenu("sidekick-tree.popupElements");
					    //loadElementPopupMenu();
					    GUIUtilities.showPopupMenu(popupElements,this,evt.getX(),evt.getY());
					  }
					  return;
					    //  System.err.println("pstiin jopa tnne, jee");
						// if(popup ==null) view.getStatus().setMessage("Popup on nulli");
						// if(popup!=null) view.getStatus().setMessage("Popup ei oo nulli");

						  //return;
						//turailu pttyy

					}


				}
				

				super.processMouseEvent(evt);
				break; //}}}
			//{{{ MOUSE_EXITED...
			case MouseEvent.MOUSE_EXITED:
				view.getStatus().setMessage(null);
				super.processMouseEvent(evt);
				break; //}}}
			default:
				super.processMouseEvent(evt);
				break;
			}
		}
	} //}}}
       
	/*//{{{ loadElementPopupMenu method
        //JR added 7.7......vasta suunnitteilla 
	private void loadElementPopupMenu(List allowedElements)
	{
	  data = SideKickParsedData.getParsedData(view);
	  if(data instanceof XMLParsedData) List elementList = (XMLParsedData(data)).getAllowedeElements();
	  for(int i=0; i<allowedElements.; i++){
             menuItem = new JMenuItem("element1");
	     menuItem.addActionListener(this);
             popupElements.add(menuItem);

	  }

	}
	///}}}*/

        private void deleteSelection() {
          //view.getStatus().setMessage("Painoit delete");

          TreePath path = tree.getSelectionPath();

          if(path != null)
          {
            Object value = ((DefaultMutableTreeNode)path
                   .getLastPathComponent()).getUserObject();

            if(value instanceof Asset)
            {
              Asset asset = (Asset) value;

              JEditTextArea textArea = view.getTextArea();

              // select the element in textArea
              textArea.setCaretPosition(asset.end.getOffset());
              textArea.addToSelection(
              new Selection.Range(asset.start.getOffset(), asset.end.getOffset()));
              // empty the selection
              textArea.setSelectedText("");
             }
	  }

        }

	//{{{ export() method
	/**
	* Save picture as png to given file
	* @param fileName
	* @param _area
	* JR added 14.5.2004
	*/
	public void export(){
		ExampleFileFilter fil = new ExampleFileFilter("png","png file");
		fc = new JFileChooser();
		fc.setFileFilter(fil);
		fc.setVisible(true);
		fc.setDialogTitle("Export drawing... ");
		fc.setApproveButtonText("Export");

		int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			file = fc.getSelectedFile();
			//this is where a real application would open the file.
			String fileName = file.getPath();
			int w = treepanel.getWidth();
			int h = treepanel.getHeight();
			BufferedImage i = new BufferedImage(w,h, BufferedImage.TYPE_INT_BGR );
			treepanel.paint(i.getGraphics());
			byte[] pngbytes;
			PngEncoderB png = new PngEncoderB(i, false, PngEncoderB.FILTER_NONE, 5);
			try {
				FileOutputStream outfile = new FileOutputStream( fileName );
				pngbytes = png.pngEncode();
				if (pngbytes == null) {
					System.out.println("Null image");
				} else {
					outfile.write( pngbytes );
				}
				outfile.flush();
				outfile.close();
			}
			catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			//        log.append("Open command cancelled by user." + newline);
		}


	} //}}}


	 // testailuja
	       public void addElement(String element){
                 //System.err.println("kummaa");
                 View view = jEdit.getActiveView();
		 TreePath path = tree.getSelectionPath();
                 if(path == null){
	           System.err.println("path oli null");
	           return;
		 }

                 Object value = ((DefaultMutableTreeNode)path
	        	.getLastPathComponent()).getUserObject();

		/*if( !(value instanceof GhostElement) ){
		  System.err.println("ei ollu ghost");
		  return;
		}*/
		//GhostElement ghostelement = (GhostElement)value;
                //int offset = ghostelement.getOffsetStart();

                JEditTextArea textArea = view.getTextArea();
                textArea.getBuffer().insert(textArea.getCaretPosition(), "<"+element+"></"+element+">");
		textArea.setCaretPosition(textArea.getCaretPosition()-5-2*(element.length()));
		sendEditTagMsg();
	       }

	       // sotkeminen pttyy



	//{{{ ActionHandler class
	class ActionHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent evt)
		{
			SideKickPlugin.parse(view,true);
		}
	} //}}}

	//{{{ CaretHandler class
	class CaretHandler implements CaretListener
	{
		public void caretUpdate(CaretEvent evt)
		{
			if(evt.getSource() == view.getTextArea() && treeFollowsCaret)
				expandTreeWithDelay();
		}
	} //}}}

	//{{{ KeyHandler class
	class KeyHandler extends KeyAdapter
	{
		public void keyPressed(KeyEvent evt)
		{
			if(caretTimer != null)
				caretTimer.stop();

                        /* JL: added VK_DELETE */
			if(evt.getKeyCode() == KeyEvent.VK_DELETE) {
                          evt.consume();
                          deleteSelection();

                        }
                        /*   ----------------  */

			if(evt.getKeyCode() == KeyEvent.VK_ENTER)
			{
				evt.consume();

				TreePath path = tree.getSelectionPath();

				if(path != null)
				{
					Object value = ((DefaultMutableTreeNode)path
						.getLastPathComponent()).getUserObject();

					if(value instanceof Asset)
					{
						Asset asset = (Asset)value;

						JEditTextArea textArea = view.getTextArea();

						if(evt.isShiftDown())
						{
							textArea.setCaretPosition(asset.end.getOffset());
							textArea.addToSelection(
								new Selection.Range(
									asset.start.getOffset(),
									asset.end.getOffset()));
						}
						else
							textArea.setCaretPosition(asset.start.getOffset());
					}
				}
			}
		}
	} //}}}

	//{{{ MouseHandler class
	class MouseHandler extends MouseMotionAdapter
	{
		public void mouseMoved(MouseEvent evt)
		{
			TreePath path = tree.getPathForLocation(
				evt.getX(),evt.getY());
			if(path == null)
				view.getStatus().setMessage(null);
			else
			{
				Object value = ((DefaultMutableTreeNode)path
					.getLastPathComponent()).getUserObject();

				if(value instanceof Asset)
				{
					view.getStatus().setMessage(((Asset)value)
						.getLongString());
				}
			}
		}
	} //}}}

	//{{{ Renderer class
	class Renderer extends DefaultTreeCellRenderer
	{
		public Component getTreeCellRendererComponent(JTree tree,
			Object value, boolean sel, boolean expanded,
			boolean leaf, int row, boolean hasFocus)
		{
			super.getTreeCellRendererComponent(tree,value,sel,
				expanded,leaf,row,hasFocus);

			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
			Object nodeValue = node.getUserObject();
			if(nodeValue instanceof Asset)
			{
				Asset asset = (Asset)node.getUserObject();

				setText(asset.getShortString());
				setIcon(asset.getIcon());
			}

			// is root?
			else if(node.getParent() == null)
			{
				setIcon(org.gjt.sp.jedit.browser.FileCellRenderer
					.fileIcon);
			}
			else
				setIcon(null);

                       /****************JL: Added**************************/
                       if (nodeValue instanceof GhostElement) {
                         setFont( getFont().deriveFont(Font.ITALIC) );
                       } else setFont( getFont().deriveFont(Font.PLAIN) );
                       /***************************************************/


			return this;
		}
	} //}}}

              //{{{ ContentAreaFocusHandler class
              /**
               *
               * <p>Title: ContentAreaFocusChanged class</p>
               * <p>Description: This class adds the text to the tree when focus of
               * the contentArea is changed</p>
               *
               *
               * @author OP
               * @version 1.0
               */
              class ContentAreaFocusAdapter extends FocusAdapter{
                 private CustomTree tree;
                 private JEditorPane contentArea;
                 public ContentAreaFocusAdapter(CustomTree tree, JEditorPane contentArea) {
                   this.tree = tree;
                   this.contentArea = contentArea;
                 }

                 //{{{ focusGained() method
                 /** OP: This method makes sure that caret is in the right place
                  *      when focus is gained
                  *
                  * NOTE!: the text is added in the MyTreeSelectionListener class
                  *
                  * @param e FocusEvent
                  */
                 public void focusGained(FocusEvent e) {
                   if ( contentArea.getText() == null ) return;
                   TreePath path = tree.getSelectionPath();
                   if ( path == null ) {
                     contentArea.setText("");
                     return;
                   }
                   contentArea.getCaret().setDot(contentArea.getText().length());
                 }//}}}

                 //{{{ focusLost() method
                 /** OP: This method implements the actions of the contentarea
                  *
                  * @param e FocusEvent
                  */
                 public void focusLost(FocusEvent e) {
                   TreePath path = tree.getSelectionPath();
                   if ( path == null ) return;
                   Object element = ((DefaultMutableTreeNode)path.getLastPathComponent()).getUserObject();
                   if ( element == null ) return;
                   if ( element instanceof Asset ) return;
                   // OP: if selected element is textelement and if the text is
                   //     changed we change the new text to buffer
                   if ( element instanceof TextElement ) {
                     TextElement text = (TextElement)element;
                     if ( (text.text).equals(contentArea.getText()) ) return;
                     if ( !((contentArea.getSelectionStart())==(contentArea.getSelectionEnd())) ) return;
                     jEdit.getActiveView().getBuffer().remove(text.start,text.end-text.start);
                     jEdit.getActiveView().getBuffer().insert(text.start,contentArea.getText());
                     return;
                   }
                   //OP: if the selected element is ghostelement and user has added
                   //    text we add the text to buffer
                   if ( element instanceof GhostElement ) {
                     GhostElement ghost = (GhostElement)element;
                     if (contentArea.getText() == null) return;
                     jEdit.getActiveView().getBuffer().insert(ghost.getOffsetStart(),contentArea.getText());
                   }
                   tree.requestFocus();
                 }//}}}

               }//}}}


               //{{{ ContentAreaCaretListener class
               /** OP: this class adds the selection in contentArea to buffer
                *
                */
               class ContentAreaCaretListener implements CaretListener {
                 private CustomTree tree;
                 private JEditorPane contentArea;

                 public ContentAreaCaretListener(CustomTree tree, JEditorPane contentArea) {
                   this.tree = tree;
                   this.contentArea = contentArea;
                 }

                 public void caretUpdate(CaretEvent e) {
                   TreePath path = tree.getSelectionPath();
                   if ( path == null ) return;
                   Object element = ((DefaultMutableTreeNode)path.getLastPathComponent()).getUserObject();
                   if ( element == null ) return;
                   //OP: here we dont do anything
                   if ( !(element instanceof TextElement) ) return;

                   TextElement textElement = (TextElement)element;

                   // Tss pit laskea selection
                   int offsetStart, offsetEnd;
                   offsetStart = (e.getMark() <  e.getDot() ? e.getMark() : e.getDot());
                   offsetEnd   = (e.getMark() >= e.getDot() ? e.getMark() : e.getDot());
                   offsetStart += textElement.start;
                   offsetEnd   += textElement.start;

                   if ( contentArea.isFocusOwner() )
                        EditBus.send(new UpdateTagListMsg(jEdit.getActiveView(),offsetStart,offsetEnd));
                 }

               }//}}}


               /**
                * Class to be used as Handler when parsing the clipboard contents that are to
                * be pasted using treeview's paste command from context menu
                * <p>Title: </p>
                * <p>Description: </p>
                * <p>Copyright: Copyright (c) 2004</p>
                * <p>Company: </p>
                * @author not attributable
                * @version 1.0
                */
               class MyHandler extends org.xml.sax.helpers.DefaultHandler {
                 MyHandler() {
                   super();
                 }

                 public void error(SAXParseException e) throws SAXParseException {
                   throw e;
                 }
                 public void warning(SAXParseException e) throws SAXParseException {
                   throw e;
                 }
               }

               class MyTreeSelectionListener implements TreeSelectionListener {
                 JTree aTree;
                 JEditorPane aEditorPane;

                 public MyTreeSelectionListener(JTree tree, JEditorPane epane) {
                   aTree = tree;
                   aEditorPane = epane;
                 }

                 /**
                  * Bad implementation, needs refactoring
                  *
                  * TODO: Should change the focus to XmlInsert when user presses Enter?
                  *     -Fix the problem with ArrayIndexOutOfBoundsException.
                  *
                  *
                  *
                  * @param e TreeSelectionEvent
                  *
                  * 10.05.2004 JL: initial version
                  * 11.05.2004 OP, JL: only one insertion inside instead of many
                  *                    don't add nested ghosts for stand-alone-elements
                  * 12.05.2004 JL: Changed ghost elements to use GhostElement-objects as userObject
                  *                   --> the problem of removing text nodes was also solved
                  * 14.5.2004  OP: Sends the UpdateTagListMsg, JL: changed few
                  *            node.getUserObject() -statements to value at the end of the method
                  * 18.5.2004  JL, OP:
                  *              - Fixed bug with nullPointerExceptions by adding two
                  *                if ( !(newPath == null) ) -lines when removing ghostElements
                  *              - Added aEditorPane updates which rely on new TextElement-class
                  * 19.5.2004  JL: a ArrayIndexOutOfBoundsException found, surrounded with try-catch
                  *
                  *
                  */
                 public void valueChanged(TreeSelectionEvent e) {
                   TreePath oldPath = e.getOldLeadSelectionPath();
                   TreePath newPath = e.getNewLeadSelectionPath();
                   //      if (oldPath==newPath) return; // shouldn't even get an event in this case?

                 try { // ---------- remove the ghost elements from old node via oldPath --------


                   if (oldPath != null) { // oldPath can be null, if nothing selected?
                     DefaultMutableTreeNode oldNode = (DefaultMutableTreeNode) oldPath.
                         getLastPathComponent();
                     // from now on I assume that oldNode is not null, because oldPath is not null

                     // ------Remove the ghost elements inside the old node-------
                     for (int i = oldNode.getChildCount() - 1; i >= 0; i--) {
                       Object value = ( (DefaultMutableTreeNode) (oldNode.getChildAt(i))).
                           getUserObject();

                       if (value instanceof GhostElement) {
                         // don't remove the current node if a GhostNode was selected
                         if (! (newPath == null))
                           if ( ( (DefaultMutableTreeNode) newPath.getLastPathComponent()).
                               getUserObject() instanceof GhostElement)
                             if ( (GhostElement) value ==
                                 (GhostElement) ( (DefaultMutableTreeNode) newPath.
                                                 getLastPathComponent()).getUserObject())continue;

                         int indexarray[] = new int[1];
                         Object removedarray[] = new Object[1];
                         removedarray[0] = oldNode.getChildAt(i);
                         indexarray[0] = i;

                         oldNode.remove(i);

                         ( (DefaultTreeModel) (aTree.getModel())).nodesWereRemoved(oldNode,
                             indexarray, removedarray);
                       }
                     }

                     // remove before- and after-ghostelements (all GhostNode-objects that old
                     // node's parent has as child)
                     DefaultMutableTreeNode oldParent = (DefaultMutableTreeNode) oldNode.
                         getParent();
                     if (oldParent != null) {
                       for (int i = oldParent.getChildCount() - 1; i >= 0; i--) {
                         Object value = ( (DefaultMutableTreeNode) (oldParent.getChildAt(i))).
                             getUserObject();
                         if (value instanceof GhostElement) {
                           // don't remove the current node if a GhostNode was selected
                           if (! (newPath == null))
                             if ( ( (DefaultMutableTreeNode) newPath.getLastPathComponent()).
                                 getUserObject() instanceof GhostElement)
                               if ( (GhostElement) value ==
                                   (GhostElement) ( (DefaultMutableTreeNode) newPath.
                                                   getLastPathComponent()).getUserObject())continue;

                           int indexarray[] = new int[1];
                           Object removedarray[] = new Object[1];
                           removedarray[0] = oldParent.getChildAt(i);
                           indexarray[0] = i;

                           oldParent.remove(i);
                           ( (DefaultTreeModel) (aTree.getModel())).nodesWereRemoved(
                               oldParent, indexarray, removedarray);
                         }
                       }
                     }
                   }
                 } catch(java.lang.ArrayIndexOutOfBoundsException ex) {
                   // If the user has changed the buffer, it usually gives one
                   // ArrayIndexOutOfBoundsException because we are trying to remove old
                   // ghostnodes which do not exist...
                 }







                 //------ add new ghost elements to the selected node--------------


                   if (newPath == null) return;
                   if ( newPath.getPathCount() < 2 ) return; // filename selected

                   DefaultMutableTreeNode node = (DefaultMutableTreeNode)newPath.getLastPathComponent();
                   if (node == null) return;

                   //---------------------------------------------------------------
                   // OP: This sends the UpdateTagListMsg to xml plugin
                   Object value = node.getUserObject();
                   if ( value instanceof Asset ) //{System.err.print("asseettiiiiiii");
                     EditBus.send(new UpdateTagListMsg(jEdit.getActiveView(),((Asset)value).start.getOffset(),((Asset)value).end.getOffset()));//}
                   else if ( value instanceof GhostElement )
                     EditBus.send(new UpdateTagListMsg(jEdit.getActiveView(),((GhostElement)value).getOffsetStart(),((GhostElement)value).getOffsetStart()));
                   else if ( value instanceof TextElement )
                     EditBus.send(new UpdateTagListMsg(jEdit.getActiveView(),-1, -1));
                   //---------------------------------------------------------------


                   // content-editing window stuff...
                   if (value instanceof TextElement) {
                     TextElement text = (TextElement) value;
                     aEditorPane.setText(jEdit.getActiveView().getBuffer().getText(text.start, text.end - text.start));
                     // OP: we can't do this because this moves the cursor and then textarea sends the update msg
                     //     and then the selection in the tree changes to parent element
                     return;
                   }

                   aEditorPane.setText("");

                   // Check that the selected element is really an element, which we can add ghostelements to.
                   if ( !(value instanceof Asset) ) return;


                   DefaultMutableTreeNode parent = (DefaultMutableTreeNode)node.getParent();
                   int index = parent.getIndex(node);

                   if (parent != newPath.getPathComponent(0)) { // if not rootnode
                     // insert after -ghostnode
                     if (index < parent.getChildCount() ) {
                       int offset = ((Asset)value).end.getOffset();
                       parent.insert(new DefaultMutableTreeNode( new GhostElement(offset, offset, GhostElement.AFTER) ), index + 1);
                       int indexarray[] = new int[1];
                       indexarray[0] = index+1;
                       ((DefaultTreeModel)(aTree.getModel())).nodesWereInserted(parent, indexarray);
                     } else aEditorPane.setText("index>=parent.getChildCount()");

                     // insert before -ghostnode
                     if (index >= 0) {
                       int offset = ((Asset)value).start.getOffset();
                       parent.insert(new DefaultMutableTreeNode( new GhostElement(offset, offset, GhostElement.BEFORE) ), index);
                       int indexarray[] = new int[1];
                       indexarray[0] = index;
                       ((DefaultTreeModel)(aTree.getModel())).nodesWereInserted(parent, indexarray);
                     } else aEditorPane.setText("index < 0. Index = " + new Integer(index).toString());
                   }

                   // nested ghostnode
                   //   + two ifs to make sure that the node is not a stand-alone xml-element
                   if (value instanceof Asset) {
                     Asset nodeObject = (Asset)value;
                     if (  !jEdit.getActiveView().getBuffer().getText( nodeObject.end.getOffset()-2,1 ).equals("/") ) {
                       int offset = 0;
                       if ( node.isLeaf() ) {
                         offset = nodeObject.end.getOffset()-nodeObject.name.length()-3;
                       }
                       else {
                         Object obj = ( (DefaultMutableTreeNode) node.getFirstChild() ).getUserObject();
                         if ( obj instanceof Asset ) {
                           nodeObject = (Asset)obj;
                           offset = nodeObject.start.getOffset();
                         }
                         if ( obj instanceof TextElement ) {
                           TextElement textEl = (TextElement)obj;
                           offset = textEl.start;
                         }
                         if ( obj instanceof GhostElement) {
                           GhostElement ghost = (GhostElement)obj;
                           offset = ghost.getOffsetStart();
                         }
                       }

                       node.insert(new DefaultMutableTreeNode( new GhostElement(offset, offset, GhostElement.INSIDE) ), 0);
                       ( (DefaultTreeModel) (aTree.getModel())).nodeStructureChanged(node);
                     }
                   }
                 }
               }







}//}}}




